import requests

import posixpath
from urlparse import urlunparse
from urllib import urlencode

from requests.adapters import HTTPAdapter
from requests.packages.urllib3.poolmanager import PoolManager
import ssl


class TLSv1Adapter(HTTPAdapter):
    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = PoolManager(num_pools=connections,
                                       maxsize=maxsize,
                                       block=block,
                                       ssl_version=ssl.PROTOCOL_TLSv1)


def get_pin_url(token, pin, fields=[]):
    """
    Construct the URL that will obtain the data
    for the specified pin
    """
    path = posixpath.sep.join(["v1/pins", pin])
    query = {"access_token": token}
    if fields:
        query["fields"] = ",".join(fields)

    return urlunparse((
        "https", "api.pinterest.com", path,
        "", urlencode(query), ""
    ))


class APIClient(object):
    """
    This client obtains the full pin details by
    using the Pinterest API
    """

    # By default, just grab all available fields
    DEFAULT_FIELDS = ["id", "link", "url", "creator",
                      "board", "created_at", "note",
                      "color", "counts", "media",
                      "attribution", "image",
                      "metadata"]

    def __init__(self, token):
        self.token = token
        self.req = requests.Session()
        self.req.mount("https://", TLSv1Adapter())

    def get_pin(self, pin_id, fields=DEFAULT_FIELDS):
        url = get_pin_url(self.token, pin_id, fields)
        return self.req.get(url).json()



from .bot_client import BotClient
from .api_client import APIClient


class EasyClient(object):
    def __init__(self, username, password, token):
        self.bot_client = BotClient(username, password)
        self.api_client = APIClient(token)

    def search_for(self, *terms):
        pins = self.bot_client.search_for(*terms)

        def full_details():
            for p in pins:
                pin_id = p.pin_id
                if pin_id:
                    yield self.api_client.get_pin(pin_id)["data"]

        return full_details()




from .widgets import LoginPage, SearchPage


class Bot(object):
    def __init__(self, session):
        self.session = session
        self.__offset = 0
        self.__pins = []

    @property
    def login_page(self):
        return LoginPage(self.session)

    @property
    def search_page(self):
        return SearchPage(self.session)

    def authenticate(self, username, password):
        lp = self.login_page
        self.session.get(lp.url)
        lp.authenticate(username, password)
        return self

    def search_for(self, *terms):
        sp = self.search_page
        sp.search_for(*terms)
        return self

    def quit(self):
        self.session.quit()

    def next(self):
        while not self.__pins:
            self.search_page.load_more_pins()
            self.__pins = self.search_page.get_pins(offset=self.__offset)
            self.__offset += len(self.__pins)

        return self.__pins.pop(0)

    def __iter__(self):
        return self



from selenium.common.exceptions import NoSuchElementException
from time import sleep

# FYI: Might need to adjust this for Python 3 compatibility
from urlparse import urlparse
import posixpath

# HACK: This is used when dealing w/ long page loads
DELAY = 10


class LoginPage(object):
    """
    Encapsulates the logic for the login page.
    """
    url = "https://www.pinterest.com/login/"

    def __init__(self, session):
        self.session = session

    @property
    def username_field(self):
        return self.session.find_element_by_name("username_or_email")

    @property
    def username(self):
        return self.username_field.get_attribute("value")

    @username.setter
    def username(self, value):
        self.username_field.clear()
        self.username_field.send_keys(value)

    @property
    def password_field(self):
        return self.session.find_element_by_name("password")

    @property
    def password(self):
        return self.password_field.get_attribute("value")

    @password.setter
    def password(self, value):
        self.password_field.clear()
        self.password_field.send_keys(value)

    @property
    def login_button(self):
        return self.session.find_element_by_css_selector("button[type='submit']")

    def submit(self):
        self.login_button.click()

    def authenticate(self, username, password):
        self.username = username
        self.password = password
        self.submit()


class SearchPage(object):
    def __init__(self, session):
        self.session = session

    @property
    def search_field(self):
        return self.session.find_element_by_name("q")

    def search_for(self, *terms):
        sf = self.search_field
        sf.clear()
        sf.send_keys(" ".join(terms) + "\n")

    def load_more_pins(self):
        self.session.execute_script("scroll(0, document.body.scrollHeight);")

        # HACK: Give the page plenty of time to load
        sleep(DELAY)
        return self

    def get_pins_query(self, offset=1, limit=None):
        query = ["//div[contains(@class, 'item')"]
        if offset > 0:
            query.append(" and position() >= %i" % (offset + 1, ))

        if limit:
            query.append(" and position() < %i" % (offset + limit + 1, ))

        query.append("]")
        return ''.join(query)

    def get_pin_elements(self, *args, **kwargs):
        query = self.get_pins_query(*args, **kwargs)
        return self.session.find_elements_by_xpath(query)

    def get_pins(self, *args, **kwargs):
        return [Pin(e) for e in self.get_pin_elements(*args, **kwargs)]


def text_for(element):
    return element.text if element else None


def extract_pin_id(url):
    parsed = urlparse(url)
    path = posixpath.normpath(parsed.path)
    components = path.split(posixpath.sep)
    if components:
        return components[-1]


class Pin(object):
    def __init__(self, base):
        self.base = base

    def get_css(self, query):
        try:
            return self.base.find_element_by_css_selector(query)
        except NoSuchElementException:
            return None

    @property
    def pinterest_link_element(self):
        return self.get_css(".pinImageWrapper")

    @property
    def pinterest_url(self):
        u = self.pinterest_link_element
        return u.get_attribute('href') if u else None

    @property
    def pin_id(self):
        u = self.pinterest_url
        return extract_pin_id(u) if u else None

    @property
    def site_link_element(self):
        return self.get_css(".pinNavLink")

    @property
    def site_url(self):
        s = self.site_link_element
        return s.get_attribute('href') if s else None

    @property
    def image_element(self):
        return self.get_css(".pinImg")

    @property
    def image_url(self):
        i = self.image_element
        return i.get_attribute("src") if i else None

    @property
    def title_element(self):
        return self.get_css(".richPinGridTitle")

    @property
    def title(self):
        return text_for(self.title_element)

    @property
    def description_element(self):
        return self.get_css(".pinDescription")

    @property
    def description(self):
        return text_for(self.description_element)

    @property
    def repin_count_element(self):
        return self.get_css(".repinCountSmall")

    @property
    def repin_count(self):
        c = self.repin_count_element
        return int(c.text) if c else 0

    @property
    def like_count_element(self):
        return self.get_css(".likeCountSmall")

    @property
    def like_count(self):
        c = self.like_count_element
        return int(c.text) if c else 0

    @property
    def comment_count_element(self):
        return self.get_css(".commentCountSmall")

    @property
    def comment_count(self):
        c = self.comment_count_element
        return int(c.text) if c else 0




from setuptools import setup, find_packages

setup(
    name = 'pintium',
    version = '1.1.0',
    description = 'Pinterest + Selenium = Pintium',
    author = 'Brian Lauber',
    author_email = 'constructible.truth@gmail.com',
    packages = find_packages(exclude = ["tests"]),
    install_requires = ["selenium", "requests"],
    test_suite = 'tests',
    tests_require = ["mock>=1.0.0"]
)



from contextlib import contextmanager
from selenium import webdriver

from ..bot import Bot

class BotClient(object):
    """
    The BotClient is useful for obtaining most
    of the pin data, including the pin id. But,
    it is currently unable to obtain the create_at
    timestamp.  For that, just use the API client.
    """
    def __init__(self, username, password):
        self.username = username
        self.password = password

    def create_bot(self, session):
        return Bot(session).authenticate(self.username, self.password)

    @contextmanager
    def session(self):
        session = webdriver.Firefox()
        try:
            yield self.create_bot(session)
        finally:
            session.quit()

    def search_for(self, *terms):
        """
        WARNING: This method is deprecated.  Please use the
                 session() contextmanager instead.
        """
        bot = self.create_bot(webdriver.Firefox())
        bot.search_for(*terms)
        return bot
